import { echo } from "@atums/echo";
import { validateJWTConfig, validateMailerConfig } from "#lib/validation";
import { isValidUrl } from "#lib/validation/url";
import { cassandraConfig, validateCassandraConfig } from "./database/cassandra";
import { jwt } from "./jwt";
import { mailerConfig } from "./mailer";

import type { Environment } from "#types/config";

const environment: Environment = {
	port: Number.parseInt(process.env.PORT || "8080", 10),
	host: process.env.HOST || "0.0.0.0",
	development:
		process.env.NODE_ENV === "development" || process.argv.includes("--dev"),
	fqdn: process.env.FRONTEND_FQDN || "",
};

function verifyRequiredVariables(): void {
	const requiredVariables = [
		"HOST",
		"PORT",

		"REDIS_URL",
		"REDIS_TTL",

		"CASSANDRA_HOST",
		"CASSANDRA_PORT",
		"CASSANDRA_CONTACT_POINTS",
		"CASSANDRA_AUTH_ENABLED",
		"CASSANDRA_DATACENTER",

		"JWT_SECRET",
		"JWT_EXPIRATION",
		"JWT_ISSUER",

		"FRONTEND_FQDN",
	];

	let hasError = false;

	for (const key of requiredVariables) {
		const value = process.env[key];
		if (value === undefined || value.trim() === "") {
			echo.error(`Missing or empty environment variable: ${key}`);
			hasError = true;
		}
	}

	const validateCassandra = validateCassandraConfig(cassandraConfig);
	if (!validateCassandra.isValid) {
		echo.error("Cassandra configuration validation failed:");
		for (const error of validateCassandra.errors) {
			echo.error(`- ${error}`);
		}
		hasError = true;
	}

	const validateJWT = validateJWTConfig(jwt);
	if (!validateJWT.valid) {
		echo.error("JWT configuration validation failed:");
		echo.error(`- ${validateJWT.error}`);
		hasError = true;
	}

	const validateMailer = validateMailerConfig(mailerConfig);
	if (!validateMailer.isValid) {
		echo.error("Mailer configuration validation failed:");
		for (const error of validateMailer.errors) {
			echo.error(`- ${error}`);
		}
		hasError = true;
	}

	const urlValidation = isValidUrl(environment.fqdn, {
		requireProtocol: true,
		failOnTrailingSlash: true,
		allowedProtocols: ["http", "https"],
		allowLocalhost: environment.development,
		allowIP: environment.development,
	});

	if (!urlValidation.valid) {
		echo.error("FRONTEND_FQDN validation failed:");
		echo.error(`- ${urlValidation.error}`);
		hasError = true;
	}

	if (hasError) {
		process.exit(1);
	}
}

export { environment, verifyRequiredVariables };
